/*
 * MP4 Command Line Tagger - mp4tag
 * 
 * Copyright (c) 2003 Dan Christiansen
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#include <mp4.h>
#include <stdio.h>
#include <string.h>
#include <inttypes.h>

#define USAGE					\
  "Usage: %s <mp4file> <options>\n"		\
  "\n"						\
  "<options> can be any combination of:\n"	\
  "-e <encoder>\n"				\
  "-a <artist>\n"				\
  "-A <album>\n"				\
  "-n <name>\n"					\
  "-c <composer>\n"				\
  "-t <track>[/<total>]\n"			\
  "-d <disk>[/<total>]\n"			\
  "-y <year>\n"					\
  "-g <genre>\n"				\
  "\n"						\
  "-h show this help\n"				\
  "-L show license\n"				\
  "\n"

#define LICENSE                                                               \
  "\n"                                                               	      \
  "MP4 Command Line Tagger - mp4tag\n"					      \
  "\n"									      \
  "Copyright (c) 2003 Dan Christiansen\n"				      \
  "\n"									      \
  "This software is provided 'as-is', without any express or implied\n"	      \
  "warranty.  In no event will the authors be held liable for any damages\n"  \
  "arising from the use of this software.\n"				      \
  "\n"									      \
  "Permission is granted to anyone to use this software for any purpose,\n"   \
  "including commercial applications, and to alter it and redistribute it\n"  \
  "freely, subject to the following restrictions:\n"			      \
  "\n"									      \
  "1. The origin of this software must not be misrepresented; you must not\n" \
  "   claim that you wrote the original software. If you use this software\n" \
  "   in a product, an acknowledgment in the product documentation would be"  \
  "\n"									      \
  "   appreciated but is not required.\n"				      \
  "2. Altered source versions must be plainly marked as such, and must not be"\
  "\n"									      \
  "   misrepresented as being the original software.\n"			      \
  "3. This notice may not be removed or altered from any source distribution."\
  "\n\n"

int main(int argc, char **argv)
{
  int b;
  MP4FileHandle mp4file;

  if (argc < 2) {
    printf(USAGE, argv[0]);
    return -1;
  }

  if (argv[1][0] == '-') {
    if (argv[1][1] == 'h')
      printf(USAGE, argv[0]);
    else if (argv[1][1] == 'L')
      printf(LICENSE);

    return 0;
  }
      

  mp4file = MP4Modify(argv[1], 0, 0);
  
  if (mp4file == MP4_INVALID_FILE_HANDLE) {
    printf("Failed to open MP4 file %s\n", argv[1]);
    return -1;
  }

  argc -= 2;
  argv += 2;

  while (argc >= 2) {
    if (argv[0][0] != '-') {
      printf("Unrecognised option %s...\n", argv[0]);
      b = TRUE;
    } else {
      switch (argv[0][1]) {
      case 'L': {
	printf(LICENSE);
	b = TRUE;

	break;
      }

      case 'e': {
	printf("Setting encoder...\n");
	b = MP4SetMetadataTool(mp4file, argv[1]);

	break;
      }
      
      case 'a': {
	printf("Setting artist...\n");
	b = MP4SetMetadataArtist(mp4file, argv[1]);
      
	break;

      }
      
      case 'A': {
	printf("Setting album...\n");
	b = MP4SetMetadataAlbum(mp4file, argv[1]);
      
	break;

      } 

      case 'n': {
	printf("Setting name...\n");
	b = MP4SetMetadataName(mp4file, argv[1]);
      
	break;

      } 

      case 'c': {
	printf("Setting composer...\n");
	b = MP4SetMetadataWriter(mp4file, argv[1]);
      
	break;

      } 

      case 't': {
	printf("Setting track...\n");
	b = MP4SetMetadataTrack(mp4file, (short)atoi(argv[1]), 0);
      
	break;

      } 

      case 'd': {
	printf("Setting disk...\n");
	b = MP4SetMetadataDisk(mp4file, (short)atoi(argv[1]), 0);
      
	break;

      } 

      case 'y': {
	if (atoi(argv[1]) > 0) {
	  printf("Setting year...\n");
	  b = MP4SetMetadataYear(mp4file, argv[1]);
	} else {
	  printf("Bad year: %s\n", argv[1]);
	}
      
	break;
      }

      case 'g': {
  	printf("Setting genre...\n");
	b = MP4SetMetadataGenre(mp4file, argv[1]);
      
	break;
      }
      }
    }
    
    if (!b)
      printf("Failed to set metadata %s to %s!\n", argv[0], argv[1]);

    argc -= 2;
    argv += 2;
  }

  b = MP4Close(mp4file);
  
  if (!b)
    printf("Failed to close MP4 file\n");

  return 0;
}
